# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.post_v1developers_auth_generate_token_response import PostV1DevelopersAuthGenerateTokenResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDevelopersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def generate_developer_authentication_token(
        self, *, developer_id: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PostV1DevelopersAuthGenerateTokenResponse]:
        """
        Generates a JWT token for developer authentication

        Parameters
        ----------
        developer_id : str
            The ID of the developer

        api_key : str
            The API key for the developer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostV1DevelopersAuthGenerateTokenResponse]
            Token generated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/developers/auth/generate-token",
            method="POST",
            json={
                "developerId": developer_id,
                "apiKey": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1DevelopersAuthGenerateTokenResponse,
                    parse_obj_as(
                        type_=PostV1DevelopersAuthGenerateTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)


class AsyncRawDevelopersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def generate_developer_authentication_token(
        self, *, developer_id: str, api_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PostV1DevelopersAuthGenerateTokenResponse]:
        """
        Generates a JWT token for developer authentication

        Parameters
        ----------
        developer_id : str
            The ID of the developer

        api_key : str
            The API key for the developer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostV1DevelopersAuthGenerateTokenResponse]
            Token generated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/developers/auth/generate-token",
            method="POST",
            json={
                "developerId": developer_id,
                "apiKey": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1DevelopersAuthGenerateTokenResponse,
                    parse_obj_as(
                        type_=PostV1DevelopersAuthGenerateTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)
