# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from .types.get_v1integrations_status_response import GetV1IntegrationsStatusResponse
from .types.post_v1integrations_connect_response import PostV1IntegrationsConnectResponse
from .types.post_v1integrations_disconnect_response import PostV1IntegrationsDisconnectResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawIntegrationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_integration_status(
        self, *, entity_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetV1IntegrationsStatusResponse]:
        """
        Retrieves the status of all integrations for an entity

        Parameters
        ----------
        entity_id : str
            The ID of the entity to get integration status for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetV1IntegrationsStatusResponse]
            Integration status retrieved successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/integrations/status",
            method="GET",
            params={
                "entityId": entity_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetV1IntegrationsStatusResponse,
                    parse_obj_as(
                        type_=GetV1IntegrationsStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)

    def connect_an_integration(
        self,
        *,
        provider: str,
        entity_id: str,
        connection_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostV1IntegrationsConnectResponse]:
        """
        Initiates the connection process for a third-party integration using the Unified API

        Parameters
        ----------
        provider : str
            The integration provider (e.g., quickbooks, xero)

        entity_id : str
            The ID of the entity to connect the integration for

        connection_type : typing.Optional[str]
            The type of connection to establish (used as scope in Unified API)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostV1IntegrationsConnectResponse]
            Integration connection initiated successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/integrations/connect",
            method="POST",
            json={
                "provider": provider,
                "entityId": entity_id,
                "connectionType": connection_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1IntegrationsConnectResponse,
                    parse_obj_as(
                        type_=PostV1IntegrationsConnectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)

    def disconnect_an_integration(
        self, *, entity_id: str, integration_type: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PostV1IntegrationsDisconnectResponse]:
        """
        Disconnects an existing integration for an entity by removing it from the Unified Connections table

        Parameters
        ----------
        entity_id : str
            The ID of the entity that owns the integration

        integration_type : str
            The type of integration to disconnect (must match connectionType in Unified Connections)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostV1IntegrationsDisconnectResponse]
            Integration disconnected successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/integrations/disconnect",
            method="POST",
            json={
                "entityId": entity_id,
                "integrationType": integration_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1IntegrationsDisconnectResponse,
                    parse_obj_as(
                        type_=PostV1IntegrationsDisconnectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)


class AsyncRawIntegrationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_integration_status(
        self, *, entity_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetV1IntegrationsStatusResponse]:
        """
        Retrieves the status of all integrations for an entity

        Parameters
        ----------
        entity_id : str
            The ID of the entity to get integration status for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetV1IntegrationsStatusResponse]
            Integration status retrieved successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/integrations/status",
            method="GET",
            params={
                "entityId": entity_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetV1IntegrationsStatusResponse,
                    parse_obj_as(
                        type_=GetV1IntegrationsStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)

    async def connect_an_integration(
        self,
        *,
        provider: str,
        entity_id: str,
        connection_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostV1IntegrationsConnectResponse]:
        """
        Initiates the connection process for a third-party integration using the Unified API

        Parameters
        ----------
        provider : str
            The integration provider (e.g., quickbooks, xero)

        entity_id : str
            The ID of the entity to connect the integration for

        connection_type : typing.Optional[str]
            The type of connection to establish (used as scope in Unified API)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostV1IntegrationsConnectResponse]
            Integration connection initiated successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/integrations/connect",
            method="POST",
            json={
                "provider": provider,
                "entityId": entity_id,
                "connectionType": connection_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1IntegrationsConnectResponse,
                    parse_obj_as(
                        type_=PostV1IntegrationsConnectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)

    async def disconnect_an_integration(
        self, *, entity_id: str, integration_type: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PostV1IntegrationsDisconnectResponse]:
        """
        Disconnects an existing integration for an entity by removing it from the Unified Connections table

        Parameters
        ----------
        entity_id : str
            The ID of the entity that owns the integration

        integration_type : str
            The type of integration to disconnect (must match connectionType in Unified Connections)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostV1IntegrationsDisconnectResponse]
            Integration disconnected successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/integrations/disconnect",
            method="POST",
            json={
                "entityId": entity_id,
                "integrationType": integration_type,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostV1IntegrationsDisconnectResponse,
                    parse_obj_as(
                        type_=PostV1IntegrationsDisconnectResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response.text)
        raise ApiError(headers=dict(_response.headers), status_code=_response.status_code, body=_response_json)
